#pageName{display:block;width:420px;_width:395px;margin:0 0 30px 25px}
#zA,#zB{float:left;width:415px;clear:left;margin-left:25px;display:inline}
#zB,#zC{_height:1px;padding-top:20px}
#zC{margin-left:40px}
#zC,{float:right;width:175px;margin-left:0}
.modWrap{padding-right:2px;_padding-right:0}
#zC{margin-right:-2px;_margin-right:0}

	#zB,
	#zC {
		background-image:  url(../bar.gif);
		background-repeat: repeat-x;
	}
	
	/* H3 Paragraph Title */
	
	#body h3 {
		font-family: Verdana, sans-serif;
		font-size: 14px;
		font-weight: bold;
		color: #666666;
	}

